;;; color-theme.el --- preview and install color themes

;; Copyright (C) 1999, 2000  Jonadab the Unsightly One <jonadab@bright.net>

;; Author: Jonadab the Unsightly One <jonadab@bright.net>
;; Maintainer: Alex Schroeder <alex@gnu.org>
;; Version: 3.0.1
;; Keywords: faces

;; This file is not (yet) part of GNU Emacs.

;; This is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2, or (at your option) any later
;; version.

;; This is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
;; MA 02111-1307, USA.

;;; Commentary:

;; Sharing your current color setup:

;; If you have already invested time in customizing Emacs faces, please
;; consider sharing your current setup.  Make sure that color-theme.el
;; is in your `load-path'.  Type M-x load-library RET color-theme RET to
;; load all the functions.  Type M-x color-theme-print RET and mail the
;; result to the maintainer of this package (see above for mail addres).

;; If you want to make sure that all your customization was exported,
;; type M-x list-faces-display RET to get a list of all faces currently
;; defined.  This is the list of faces that `color-theme-print' uses.

;; Previewing and installing a color theme:

;; Make sure that color-theme.el is in your `load-path'.  Type M-x
;; load-library RET color-theme RET to load all the functions.

;; The main function to call is color-theme-select.  Type M-x
;; color-theme-select RET.  That creates a Color Theme Selection buffer
;; for selecting a theme to preview.  Press RET on a color theme to
;; preview it.  Press `i' to install the color theme for the rest of
;; your session.

;; If you want to install the color theme as soon as Emacs is started
;; up, read the description of the theme you like and remember the name
;; of the color theme function.  Press `d' on a color theme in the Color
;; Theme Selection buffer to read the description.

;; Assuming you like the Gnome2 theme, add the following to the end of
;; your .emacs (removing the leading `;;').

;; (require 'color-theme)
;; (color-theme-gnome2)

;; Deriving your own color theme:

;; If you want to derive your own color theme from an existing color
;; theme, press `p' in the Color Theme Selection buffer (it doesn't
;; matter where in the buffer you press `p'.  This creates a buffer with
;; the elisp code needed to install the current color theme.  Copy the
;; entire code to your .emacs and start fooling around with it.  Read
;; the documentation of color-theme-install using C-h f
;; color-theme-install RET.

;; Note that all color theme previews are cumulative.  You can try to
;; combine several color themes.  This makes sense if one color theme
;; defines faces which another color theme does not.  Install both
;; themes by pressing `i' on them in the Color Theme Selection buffer,
;; press `p' to get the elisp code, paste it into your .emacs and start
;; working on your masterpiece.

;; Changing menu colors:

;; If are using X, you can set the menu foreground and background using
;; your .Xdefaults file.  If you set emacs*Background and
;; emacs*Foreground, the first frame will be created with these
;; foreground and background colors used for the menu.  If your .emacs
;; loads a color theme, the frame foreground and background colors
;; overwrite the settings from the .Xdefaults file in the frame itself,
;; but not for the menu.  This assumes that you are not setting any menu
;; ressources for Emacs in the .Xdefaults file.  Here is a sample entry
;; for your .Xdefaults file:
;;   emacs*Background:		DarkSlateGray
;;   emacs*Foreground:		wheat

;;; Bugs:

;; None known.  
;; Probably XEmacs incompatible; contributions welcome.
;; Probably confuses customize-face stuff.



;;; Code:

(defvar color-theme-buffer-name "*Color Theme Selection*"
  "Name of the color theme selection buffer.")

(defvar color-theme-mode-map
  (let ((map (make-sparse-keymap)))
    (define-key map (kbd "RET") 'color-theme-preview-at-point)
    (define-key map (kbd "i") 'color-theme-install-at-point)
    (define-key map (kbd "d") 'color-theme-describe)
    (define-key map (kbd "q") 'bury-buffer)
    (define-key map (kbd "p") 'color-theme-print)
    (define-key map (kbd "?") 'color-theme-describe)
    (define-key map [mouse-2] 'color-theme-preview-at-mouse)
    (define-key map [S-mouse-2] 'color-theme-install-at-mouse)
    map)
  "Mode map used for the buffer created by `color-theme-select'.")

(defvar color-theme-legal-frame-parameters "\\(color\\|mode\\)$"
  "Regexp that matches frame parameter names.
Only frame parameter names that match this regexp can be changed as part
of a color theme.")

(defvar color-theme-backup nil
  "Original color theme used before any other color theme was installed.
See `color-theme-install' for more information on the theme data structure.
See `color-theme-uninstall' for more information about the use if this 
variable.  This variable is set by `color-theme-store-backup'.")

;;; Selection of a color-theme

(defun color-theme-select ()
  "Displays a special buffer for selecting and installing a color theme."
  (interactive)
  (switch-to-buffer (get-buffer-create color-theme-buffer-name))
  (setq buffer-read-only nil)
  (erase-buffer)
  (let ((uninstall "Uninstall: Restore original colors"))
    (put-text-property 0 (length uninstall) 'color-theme 'color-theme-uninstall uninstall)
    (put-text-property 0 9 'face 'bold uninstall)
    (put-text-property 0 9 'mouse-face 'highlight uninstall)
    (insert uninstall)
    (newline 2))
  (let ((themes color-themes))
    (while themes
      (let* ((theme (car themes))
	     (func (nth 0 theme))
	     (name (nth 1 theme))
	     (author (nth 2 theme)))
	(setq desc (format "%s by %s" name author))
	(put-text-property 0 (length desc) 'color-theme func desc)
	(put-text-property 0 (length name) 'face 'bold desc)
	(put-text-property 0 (length name) 'mouse-face 'highlight desc)
        (insert desc)
	(newline))
      (setq themes (cdr themes))))
  (beginning-of-buffer)
  (setq buffer-read-only t)
  (set-buffer-modified-p nil)
  (color-theme-mode))

(defun color-theme-mode ()
  "Major mode to select and preview color themes.

Use \\[color-theme-install-at-point] to install a color theme in the current frame
for the current session.  Note that the changes are applied on top of
your current setup.  This is a feature.  If you want to start with a
clean setup, choose the uninstall entry or use \\[color-theme-uninstall].  
See `color-theme-uninstall' about the limitations of the uninstall.

\\{color-theme-mode-map}

If you want to install this color theme permanently, use \\[color-theme-print] 
and copy the result into your .emacs file.

Note that the Emacs menu is not affected by color themes within Emacs.
Depending on the toolkit you used to compile Emacs, you might have to
set specific X ressources.  See the info manual for more information.

The color themes are listed in `color-themes', which see."
  (kill-all-local-variables)
  (setq major-mode 'color-theme-mode)
  (setq mode-name "Color Themes")
  (use-local-map color-theme-mode-map)
  (goto-address))

;;; Commands in Color Theme Selection mode

(defun color-theme-describe ()
  "Describe color theme listed at point.
This shows the documentation of the value of text-property color-theme
at point.  The text-property color-theme should be a color theme
function.  See `color-themes'."
  (interactive)
  (describe-function (get-text-property (point) 'color-theme)))

(defun color-theme-preview-at-mouse (event)
  "Preview color theme clicked upon using the mouse.
Calls `color-theme-install-at-mouse' with the EVENT argument and
the preview argument."
  (interactive "e")
  (color-theme-install-at-mouse event t))

(defun color-theme-install-at-mouse (event &optional preview)
  "Install color theme clicked upon using the mouse.
First argument EVENT is used to set point.  Then
`color-theme-install-at-point' is called with optional argument PREVIEW.
If PREVIEW is non-nil, the color theme is only previewed."
  (interactive "e")
  (save-excursion
    (let ((posn (event-start event)))
      (set-buffer (window-buffer (posn-window posn)))
      (goto-char (posn-point posn))
      (color-theme-install-at-point preview))))

(defun color-theme-preview-at-point ()
  "Preview color theme at point.
Calls `color-theme-install-at-point' with the preview argument."
  (interactive)
  (color-theme-install-at-point t))

(defun color-theme-install-at-point (&optional preview)
  "Installs color theme at point.
This calls the value of text-property color-theme at point with the
optional PREVIEW argument.  If PREVIEW is non-nil, the color theme is
only previewed.  The text-property color-theme should be a color theme
function.  See `color-themes'."
  (interactive)
  (funcall (get-text-property (point) 'color-theme) preview))

;;; Uninstallation of a color theme, backup creation

(defun color-theme-uninstall (&optional preview)
  "Restore the original color setup.
The original setup is stored in `color-theme-backup'.  The backup
variable is filled by `color-theme-store-backup'.

Note that the backup contains all face the faces defined at the time it
was created.  Usually that happens when `color-theme-install' is called
by a color theme function such as `color-theme-gnome2'.  Therefore, the
uninstall of a color theme will not be able to reset faces which were
undefined at the time the backup was stored."
  (interactive)
  (if (null color-theme-backup)
      (error "No color theme installed")
    (color-theme-install preview color-theme-backup)))

(defun color-theme-store-backup ()
  "Store original color setup in `color-theme-backup'.
The color setup is generated by `color-theme-snapshot'.  This function
called when `color-theme-install' runs for the first time and
`color-theme-backup' is nil.  The value of `color-theme-backup' is used
in `color-theme-uninstall'."
  (setq color-theme-backup (color-theme-snapshot)))

(defun color-theme-snapshot ()
  "Make a snapshot of the current color theme.
The cons cell returned can be stored in `color-theme-backup'
or used as a parameter to `color-theme-install'."
  (let* ((frame (selected-frame))
	 (backup-frame-param (color-theme-filter-frame-params (frame-parameters frame)))
	 (backup-faces-defs)
	 (faces (face-list))
	 (face))
    (while faces
      (setq face (car faces))
      (setq faces (cdr faces))
      (add-to-list 'backup-faces-defs
		   (list face
			 (face-foreground face frame)
			 (face-background face frame)
			 (face-bold-p face frame)
			 (face-italic-p face frame)
			 (face-underline-p face frame))))
    (cons backup-frame-param backup-faces-defs)))

(defun color-theme-filter-frame-params (params)
  "Filter the frame parameters in PARAMS.
The resulting alist only contains keys matching
`color-theme-legal-frame-parameters'."
  (let ((param) (new-params))
    (while params
      (setq param (car params))
      (setq params (cdr params))
      (if (string-match color-theme-legal-frame-parameters (symbol-name (car param)))
	  (add-to-list 'new-params param)))
    new-params))

;;; Installation of a color theme

(defun color-theme-install-frame-parameters (frame parameters)
  "Change frame parameters of frame FRAME using alist PARAMETERS.  
If FRAME is nil, change all frames, `initial-frame-alist', and
`default-frame-alist'.  

Called from `color-theme-install'.  See the elisp manual for more
information on frame parameters.

Example: '((foreground-color . \"white\") (background-color . \"black\"))"
  (let ((params (color-theme-filter-frame-params parameters)))
    (if frame
	(modify-frame-parameters frame params)
      (let ((frames (frame-list)))
	(while frames
	  (setq frame (car frames))
	  (setq frames (cdr frames))
	  (modify-frame-parameters frame params)))
      (setq initial-frame-alist (append params initial-frame-alist)
	    default-frame-alist (append params default-frame-alist)))))

(defun color-theme-install-faces (frame faces)
  "Change faces of frame FRAME using alist FACES.

If first argument FRAME is nil, change faces for all frames and create
any faces listed in FACES which don't exist.  If first argument FRAME is
non-nil, the faces are only updated for FRAME.  No new faces will be
created.

Called from `color-theme-install'.

Face definitions have the form
  (FACE FOREGROUND BACKGROUND BOLDP ITALICP UNDERLINEP)
where FACE is a face symbol, FOREGROUND and BACKGROUND are color names, and
the remaining entries are either nil or t.

Example: '((default \"white\" \"black\" nil nil nil)
           \(modeline \"black\" \"white\" nil nil nil))"
  (while faces
    (let* ((face-def (car faces))
	   (face (nth 0 face-def)))
      (if (or (facep face)
	      (and (null frame)
		   (make-face face)))
	  (progn
	    (set-face-foreground (nth 0 face-def) (nth 1 face-def) frame)
	    (set-face-background (nth 0 face-def) (nth 2 face-def) frame)
	    (set-face-bold-p (nth 0 face-def) (nth 3 face-def) frame)
	    (set-face-italic-p (nth 0 face-def) (nth 4 face-def) frame)
	    (set-face-underline-p (nth 0 face-def) (nth 5 face-def) frame))))
    (setq faces (cdr faces))))

(defun color-theme-install (preview theme)
  "Install a color theme defined by frame parameters and faces.

If first argument PREVIEW is non-nil, the theme is only installed for
the selected frame; all faces already defined are updated.  No new faces
are created.  If first argument PREVIEW is nil, the theme is installed
for all present and future frames; any missing faces are created.  See
`color-theme-install-faces'.

Second argument THEME is a color theme definition.  See below for more
information.

If `color-theme-backup' is nil, a backup of the current color theme is
stored in `color-theme-backup'.  See `color-theme-store-backup'.

If you want to install a color theme from your .emacs, use the output
generated by `color-theme-print'.  This produces a call to
color-theme-install which you can copy to your .emacs.

Example: \(color-theme-install nil
          '(((foreground-color . \"wheat\"))
             \(modeline \"yellow\" \"red\")
             \(region \"white\" \"red\")))
         \(setq color-theme-backup nil)

A color theme definition is a cons cell 
\(FRAME-PARAMETERS-ALIST . FACE-DEFINITION-ALIST).

FRAME-PARAMETERS-ALIST is an alist of frame parameters.  These are
installed with `color-theme-install-frame-parameters'.

FACE-DEFINITION-ALIST is an alist of face definitions.  These are
installed with `color-theme-install-faces'.

Here is a very simple example using a cons cell:

\(defun my-theme (&optional preview)
  \"Wheat background, default colors except for fancy modeline and region.\"
  \(color-theme-install preview
   '(((foreground-color . \"wheat\"))
     . \((modeline \"yellow\" \"red\")
        \(region \"white\" \"red\")))))

This is equivalent to the following notation using a list:

\(defun my-theme (&optional preview)
  \"Wheat background, default colors except for fancy modeline and region.\"
  \(color-theme-install preview
   '(((foreground-color . \"wheat\"))
     \(modeline \"yellow\" \"red\")
     \(region \"white\" \"red\"))))

This works because the only thing that has to be true about the theme is
that the cdr of the theme is the face definition alist -- which is true
in either case."
  (if (null color-theme-backup)
      (color-theme-store-backup))
  (let ((frame (if preview (selected-frame))))
    (color-theme-install-frame-parameters frame (car theme))
    (color-theme-install-faces frame (cdr theme))))

(defun color-theme-print ()
  "Print the current color theme so you can contribute it to
<URL:news:gnu.emacs.sources> or paste it into your .emacs file.

If you want to use a specific color theme function, you can call the
color theme function in your .emacs directly.

Example: \(require 'color-theme)
         \(color-theme-gnome2)
         \(setq color-theme-backup nil)"
  (interactive)
  (let ((theme (color-theme-snapshot)))
    (switch-to-buffer (get-buffer-create "*Color Theme*"))
    (setq buffer-read-only nil)
    (erase-buffer)
    (insert "(require 'color-theme)\n"
	    "(color-theme-install nil\n"
	    "  '" (format "%S" theme) ")\n"
	    "(setq color-theme-backup nil)\n")
    (emacs-lisp-mode)))



;;; List of color themes

(defvar color-themes
  '((color-theme-simple-1 "Black" "Jonadab <jonadab@bright.net>")
    (color-theme-gnome "Gnome" "Jonadab <jonadab@bright.net>")
    (color-theme-gnome2 "Gnome 2" "Alex <a.schroeder@bsiag.ch>")
    (color-theme-jonadabian "Jonadab" "Jonadab <jonadab@bright.net>")
    (color-theme-ryerson "Ryerson" "Luis Fernandes <elf@ee.ryerson.ca>")
    (color-theme-standard "Standard" "Alex <a.schroeder@bsiag.ch>")
    (color-theme-wheat "Wheat" "Alex <a.schroeder@bsiag.ch>"))
  "List of color themes.  

Each THEME is itself a three element list (FUNC NAME MAINTAINER).  

FUNC is a color theme function which does the setup.  The function FUNC
must accept a PREVIEW parameter and call `color-theme-install', passing
PREVIEW along.  The color theme function can be interactive.

NAME is the name of the theme and MAINTAINER is the name and/or email of
the maintainer of the theme.

If you defined your own color theme and want to add it to this list,
use this:

  (add-to-list 'color-themes '(color-theme-gnome2 \"Gnome2\" \"Alex\"))")

(defun color-theme-gnome (&optional preview)
  "Wheat on darkslategrey scheme one version of Emacs in RH6 and Gnome,
modified by Jonadab."
  (interactive)
  (color-theme-install preview
   '(((foreground-color . "wheat")
      (background-color . "darkslategrey")
      (background-mode . 'dark)) 
     (default "wheat" "darkslategrey")
     (region "cyan" "dark cyan")
     (underline "yellow" nil nil nil t)
     (modeline "dark cyan" "wheat")
     (italic "dark red" nil nil t)
     (bold-italic "dark red" nil t t)
     (font-lock-comment-face "Firebrick" nil nil nil nil)
     (bold nil nil t))))

(defun color-theme-gnome2 (&optional preview)
  "Wheat on darkslategrey scheme.  `color-theme-gnome' started it all.
On my NT Emacs with a true color display it looks very somber and dark:
grey background, lots of faces use wheat, khaki, tomato and coral.  On
my Linux I only get to choose from 64 colors and it still looks Ok.  I
don't use Gnus on my Linux, thowgh, so I don't know about the Gnus faces.
If you are using this color theme and have some additional faces to add
for modes I don't use, please share them with me!"
  (interactive)
  (color-theme-install preview
   '(((foreground-color . "wheat")
      (background-color . "darkslategrey")
      (cursor-color . "LightGray")
      (mouse-color . "Grey")
      (background-mode . 'dark)) 
     (default "wheat" "darkslategrey")
     (region "cyan" "dark cyan")
     (highlight "DarkGreen" "PaleGreen")
     (secondary-selection "SlateBlue" "Aquamarine")
     (show-paren-match-face "SlateBlue" "Aquamarine")
     (modeline "wheat" "DarkOliveGreen")
     (font-lock-keyword-face "Salmon")
     (font-lock-type-face "YellowGreen")
     (font-lock-comment-face "IndianRed")
     (font-lock-function-name-face "MediumSlateBlue")
     (eshell-ls-directory-face "MediumSlateBlue" nil t) 
     (eshell-ls-symlink-face "White")
     (eshell-ls-executable-face "OrangeRed")
     (eshell-ls-readonly-face "Turquoise")
     (eshell-ls-unreadable-face "DimGray")
     (eshell-ls-special-face "Gold")
     (eshell-ls-archive-face "IndianRed" nil t)
     (eshell-ls-backup-face "Grey")
     (eshell-ls-picture-face "Violet")
     (eshell-ls-clutter-face "DimGray")
     (eshell-prompt-face nil nil t)
     (info-node "DodgerBlue1" nil t nil t)
     (info-xref "DodgerBlue1" nil nil nil t)
     (message-header-to-face "light cyan" nil t)
     (message-header-cc-face "light cyan")
     (message-header-subject-face "light cyan" nil t)
     (message-header-newsgroups-face "LightSkyBlue3" nil t t)
     (message-header-other-face "LightSkyBlue3")
     (message-header-name-face "DodgerBlue1")
     (message-header-xheader-face "DodgerBlue3")
     (message-separator-face "chocolate")
     (gnus-header-content-face "LightSkyBlue3")
     (gnus-header-from-face "light cyan" nil t)
     (gnus-header-name-face "DodgerBlue1" nil t)
     (gnus-header-newsgroups-face "LightSkyBlue3" nil t t)
     (gnus-header-subject-face "light cyan" nil t)
     (message-cited-text-face "tomato")
     (gnus-cite-face-1 "tomato")
     (gnus-cite-face-2 "dark khaki")
     (gnus-cite-face-3 "olive drab")
     (gnus-signature-face "salmon")
     (gnus-group-news-1-face "tomato" nil t)
     (gnus-group-news-1-empty-face "tomato")
     (gnus-group-news-2-face "dark orange" nil t)
     (gnus-group-news-2-empty-face "dark orange")
     (gnus-group-news-3-face "green yellow" nil t)
     (gnus-group-news-3-empty-face "green yellow")
     (gnus-group-news-low-face "yellow green" nil t)
     (gnus-group-news-low-empty-face "yellow green")
     (gnus-group-mail-1-face "tomato" nil t)
     (gnus-group-mail-1-empty-face "tomato")
     (gnus-group-mail-2-face "light cyan" nil t)
     (gnus-group-mail-2-empty-face "light cyan")
     (gnus-group-mail-3-face "turquoise" nil t)
     (gnus-group-mail-3-empty-face "turquoise")
     (gnus-group-mail-low-face "dodger blue" nil t)
     (gnus-group-mail-low-empty-face "dodger blue")
     (gnus-summary-selected-face nil nil nil nil t)
     (gnus-summary-cancelled-face "yellow" "black")
     (gnus-summary-high-ticked-face "burlywood" nil t)
     (gnus-summary-low-ticked-face "chocolate" nil nil t)
     (gnus-summary-normal-ticked-face "sandy brown")
     (gnus-summary-high-ancient-face "RoyalBlue" nil t)
     (gnus-summary-normal-ancient-face "RoyalBlue")
     (gnus-summary-low-ancient-face "chocolate" nil nil t)
     (gnus-summary-high-unread-face "cyan" nil t)
     (gnus-summary-normal-unread-face "aquamarine")
     (gnus-summary-low-unread-face "light sea green" nil nil t)
     (gnus-summary-high-read-face "forest green" nil t)
     (gnus-summary-normal-read-face "khaki")
     (gnus-summary-low-read-face "light sea green")
     (gnus-splash-face "ForestGreen")
     (custom-button-face "MediumSlateBlue" nil nil nil t)
     (custom-documentation-face "Grey")
     (custom-group-tag-face "MediumAquamarine")
     (custom-variable-tag-face "Aquamarine")
     (custom-state-face "Coral")
     (widget-field-face "LightBlue")
     (widget-single-line-field-face "LightBlue")
     (diary-face "IndianRed" nil t))))

(defun color-theme-simple-1 (&optional preview)
  "Black bg; doesn't mess with most faces, but does turn on dark
background mode."
  (interactive)
  (color-theme-install preview
   '(((foreground-color . "white")
      (background-color . "black")
      (cursor-color     . "indian red")
      (background-mode  . 'dark))
     (default "white" "black")
     (modeline "black" "white")
     (underline nil nil nil nil t)
     (region nil "grey"))))

(defun color-theme-jonadabian (&optional preview)
  "Dark blue bg; specifies fg for most faces; default fg is similar to
wheat."
  (interactive)
  (color-theme-install preview
   '(((foreground-color . "wheat")
      (cursor-color . "medium turquoise")
      (background-color . "#000055")
      (background-mode . 'dark))
     (default "#CCBB77" "#000055")
     (modeline "cyan" "#007080")
     (underline nil nil nil nil t)
     (region nil "#004080")
     (font-lock-keyword-face "#00BBBB")
     (font-lock-comment-face "grey50" nil t t)
     (font-lock-string-face "#10D010")
     (font-lock-constant-face "indian red")
     (highlight-changes-face nil "navy")
     (highlight-changes-delete-face "red" "navy")
     (widget-field-face "black" "grey35")
     (widget-inactive-face "gray")
     (custom-button-face "yellow" "dark blue")
     (custom-state-face "mediumaquamarine")
     (custom-face-tag-face "goldenrod" nil nil nil t)
     (custom-documentation-face "#10D010")
     (custom-set-face "#2020D0"))))

(defun color-theme-ryerson (&optional preview)
  "White on midnightblue scheme used at Ryerson Polytechnic University
in the Electronic Engineering department."
  (interactive)
  (color-theme-install preview
   '(((foreground-color . "white")
      (background-color . "midnightblue")
      (cursor-color     . "red")
      (background-mode  . 'dark))
     (default "white" "midnightblue")
     (modeline "black" "slategray3")
     (underline nil nil nil nil t)
     (region "black" "slategray3"))))

(defun color-theme-wheat (&optional preview)
  "Default colors on a wheat background."
  (interactive)
  (color-theme-install preview
   '(((foreground-color . "black") 
      (background-color . "wheat")
      (mouse-color . "black") 
      (cursor-color . "black") 
      (border-color . "black")
      (background-mode . light))
     (modeline "wheat" "black")
     (highlight nil "darkseagreen2")
     (region nil "Tan")
     (secondary-selection nil "paleturquoise")
     (show-paren-match-face nil "turquoise")
     (show-paren-mismatch-face "white" "purple")
     (font-lock-comment-face "Firebrick")
     (font-lock-string-face "DimGrey")
     (font-lock-keyword-face "Purple")
     (font-lock-builtin-face "Orchid")
     (font-lock-function-name-face "Blue")
     (font-lock-variable-name-face "DarkGoldenrod")
     (font-lock-type-face "ForestGreen")
     (font-lock-constant-face "CadetBlue")
     (font-lock-warning-face "Red" nil t))))

(defun color-theme-standard (&optional preview)
  "The default colors.  Use this theme to undo it all.
I'm interested in adding faces from the standard Emacs
distribution to this but too lazy to extract them from
the source.  If you are missing some faces here, please
notify me."
  (interactive)
  (color-theme-install preview
   '(((foreground-color . "black")
      (background-color . "white")
      (mouse-color . "black")
      (cursor-color . "black")
      (border-color . "black")
      (background-mode . light)) 
     (default "black" "white")
     (modeline "white" "black")
     (highlight nil "darkseagreen2")
     (bold nil nil t)
     (italic nil nil nil t)
     (bold-italic nil nil t t)
     (region nil "gray")
     (secondary-selection nil "paleturquoise")
     (underline nil nil nil nil t)
     (show-paren-match-face nil "turquoise")
     (show-paren-mismatch-face "white" "purple")
     (widget-documentation-face "dark green")
     (widget-button-face nil nil t)
     (widget-field-face nil "gray85")
     (widget-single-line-field-face nil "gray85")
     (widget-inactive-face "dim gray")
     (widget-button-pressed-face "red")
     (custom-invalid-face "yellow" "red")
     (custom-rogue-face "pink" "black")
     (custom-modified-face "white" "blue")
     (custom-set-face "blue" "white")
     (custom-changed-face "white" "blue")
     (custom-saved-face nil nil nil nil t)
     (custom-state-face "dark green")
     (custom-variable-tag-face "blue" nil nil nil t)
     (custom-variable-button-face nil nil t nil t)
     (custom-face-tag-face nil nil nil nil t)
     (custom-group-tag-face-1 "red" nil nil nil t)
     (custom-group-tag-face "blue" nil nil nil t)
     (message-header-to-face "MidnightBlue" nil t)
     (message-header-cc-face "MidnightBlue")
     (message-header-subject-face "navy blue" nil t)
     (message-header-newsgroups-face "blue4" nil t t)
     (message-header-other-face "steel blue")
     (message-header-name-face "cornflower blue")
     (message-header-xheader-face "blue")
     (message-separator-face "brown")
     (message-cited-text-face "red")
     (gnus-group-news-1-face "ForestGreen" nil t)
     (gnus-group-news-1-empty-face "ForestGreen")
     (gnus-group-news-2-face "CadetBlue4" nil t)
     (gnus-group-news-2-empty-face "CadetBlue4")
     (gnus-group-news-3-face nil nil t)
     (gnus-group-news-3-empty-face)
     (gnus-group-news-low-face "DarkGreen" nil t)
     (gnus-group-news-low-empty-face "DarkGreen")
     (gnus-group-mail-1-face "DeepPink3" nil t)
     (gnus-group-mail-1-empty-face "DeepPink3")
     (gnus-group-mail-2-face "HotPink3" nil t)
     (gnus-group-mail-2-empty-face "HotPink3")
     (gnus-group-mail-3-face "magenta4" nil t)
     (gnus-group-mail-3-empty-face "magenta4")
     (gnus-group-mail-low-face "DeepPink4" nil t)
     (gnus-group-mail-low-empty-face "DeepPink4")
     (gnus-summary-selected-face nil nil nil nil t)
     (gnus-summary-cancelled-face "yellow" "black")
     (gnus-summary-high-ticked-face "firebrick" nil t)
     (gnus-summary-low-ticked-face "firebrick" nil nil t)
     (gnus-summary-normal-ticked-face "firebrick")
     (gnus-summary-high-ancient-face "RoyalBlue" nil t)
     (gnus-summary-low-ancient-face "RoyalBlue" nil nil t)
     (gnus-summary-normal-ancient-face "RoyalBlue")
     (gnus-summary-high-unread-face nil nil t)
     (gnus-summary-low-unread-face nil nil nil t)
     (gnus-summary-normal-unread-face)
     (gnus-summary-high-read-face "DarkGreen" nil t)
     (gnus-summary-low-read-face "DarkGreen" nil nil t)
     (gnus-summary-normal-read-face "DarkGreen")
     (gnus-splash-face "ForestGreen")
     (info-node nil nil t t)
     (info-menu-5 nil nil nil nil t)
     (info-xref nil nil t)
     (font-lock-comment-face "Firebrick")
     (font-lock-string-face "RosyBrown")
     (font-lock-keyword-face "Purple")
     (font-lock-builtin-face "Orchid")
     (font-lock-function-name-face "Blue")
     (font-lock-variable-name-face "DarkGoldenrod")
     (font-lock-type-face "ForestGreen")
     (font-lock-constant-face "CadetBlue")
     (font-lock-warning-face "Red" nil t))))

(provide 'color-theme)

;;; colors.el ends here
